package com.cicdi.notify;

import com.cicdi.notify.template.Template;
import com.cicdi.notify.template.TemplateManager;

import java.util.Map;

/**
 * 通知器抽象实现
 * <p>实现了通用的发送方法，还需要具体服务商继承此类，实现全部的方法</p>
 *
 * @author xueye
 */
public abstract class AbstractNotifier<T extends Template> implements Notifier<T> {

    /**
     * 通知模板管理器，用于获取当前通知类型使用的通知模板
     */
    private final TemplateManager templateManager;

    public AbstractNotifier(TemplateManager templateManager) {
        this.templateManager = templateManager;
    }

    @Override
    public void send(String templateId, Map<String, Object> context) {
        Template template = templateManager.getTemplate(this.getType(), templateId);
        if (null == template) {
            throw new UnsupportedOperationException("指定的模版不存在:" + templateId);
        }
        this.send((T) template, context);
    }


}
