package com.cicdi.notify.template;

import com.cicdi.notify.NotifyType;

import java.util.concurrent.ConcurrentHashMap;

/**
 * 通知模板管理器
 * <p>通知模板的转换在不同的通知服务商{@link TemplateProvider}中实现</p>
 *
 * @author xueye
 * @see Template
 * @see TemplateProvider
 */
public interface TemplateManager {

    /**
     * 根据ID获取通知模板
     * <p>使用容器缓存模板，如果不存在调用{@link #createTemplate(NotifyType, TemplateProperties)}方法创建，
     * 并添加到容器中，容器建议使用线程安全的{@link ConcurrentHashMap}集合</p>
     *
     * @param type       通知类型
     * @param templateId 通知模板ID
     * @return 通知模板
     */
    Template getTemplate(NotifyType type, String templateId);

    /**
     * 使用提供的模板配置属性创建通知模板
     * <p>根据不同的通知类型处理对应的模板</p>
     *
     * @param type       通知类型
     * @param properties 通知模板属性
     * @return 通知模板
     */
    Template createTemplate(NotifyType type, TemplateProperties properties);

    /**
     * 注册通知服务商
     * <p>服务商经过注册之后才可以使用, 如果通知模板配置的服务商不在已注册集合中, 将会抛出{@link UnsupportedOperationException}</p>
     *
     * @param provider 通知服务商
     * @see UnsupportedOperationException
     */
    void register(TemplateProvider provider);

    /**
     * 重新加载通知模板
     * <p>如果通知模板属性有变更，调用此方法更新容器缓存中的通知模板</p>
     *
     * @param templateId 通知模板ID
     */
    void reload(String templateId);

}
