package com.cicdi.notify.util;

import com.cicdi.notify.util.script.engine.DynamicScriptEngine;
import com.cicdi.notify.util.script.engine.spel.SpelParserEngine;
import org.apache.commons.beanutils.BeanUtilsBean2;
import org.apache.commons.codec.digest.DigestUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * 表达式工具,用户解析表达式为字符串
 */
public class ExpressionUtils {
    //表达式提取正则 ${.+?}
    private static final Pattern PATTERN = Pattern.compile("(?<=\\$\\{)(.+?)(?=})");

    /**
     * 获取默认的表达式变量
     *
     * @return 变量集合
     */
    public static Map<String, Object> getDefaultVar() {
        return new HashMap<>();
    }

    /**
     * 获取默认的表达式变量并将制定的变量合并在一起
     *
     * @param var 要合并的变量集合
     * @return 变量集合
     */
    public static Map<String, Object> getDefaultVar(Map<String, Object> var) {
        Map<String, Object> vars = getDefaultVar();
        vars.putAll(var);
        return vars;
    }

    /**
     * 使用默认的变量解析表达式
     *
     * @param expression 表达式字符串
     * @return 解析结果
     * @throws Exception 解析错误
     * @see ExpressionUtils#analytical(String, Map)
     */
    public static String analytical(String expression) throws Exception {
        return analytical(expression, new HashMap<>());
    }

    /**
     * 解析表达式,表达式使用{@link ExpressionUtils#PATTERN}进行提取<br>
     * 如调用 analytical("http://${3+2}/test",var,"spel")<br>
     *
     * @param expression 表达式字符串
     * @param context    变量
     * @return 解析结果
     */
    public static String analytical(String expression, Map<String, Object> context) {
        if (!expression.contains("${")) {
            return expression;
        }

        return TemplateParser.parse(expression, key -> {
            if (key == null || "".equals(key)) {
                return "";
            }

            // SpEL解析
            DynamicScriptEngine engine = new SpelParserEngine();
            String id = DigestUtils.md5Hex(key);
            try {
                if (!engine.compiled(id)) {
                    engine.compile(id, key);
                }
                if (engine.execute(id, context).isSuccess()) {
                    return String.valueOf(engine.execute(id, context).getIfSuccess());
                }
            } catch (Exception e) {
                return "";
            }

            // 表达式替换
            if (!key.contains("#")) {
                try {
                    Object fast = BeanUtilsBean2.getInstance().getPropertyUtils().getProperty(context, key);
                    if (fast != null) {
                        return fast.toString();
                    }
                } catch (Exception ignore) {
                    return "";
                }
            }
            return "";
        });
    }

}
